import datetime
import uuid

from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from apps.financial.models import SepaMandate
from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import AssociationSpecificDataFields
from apps.MySU.serializers.association import AssociationSerializerList
from apps.MySU.serializers.association_specific_data import AssociationSpecificDataSerializerSimple
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.membertype import MemberTypeSerializer
from apps.MySU.serializers.profile import ProfileSerializer
from MySU.helper_functions import get_for_patch


class AssociationMembershipSerializerList(BaseSerializer):
    class Meta:
        model = AssociationMembership
        fields = ('url', 'profile', 'association')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'}
        }

class AssociationMembershipSerializerBase(FlexFieldsModelSerializer, serializers.HyperlinkedModelSerializer):
    specific_data = AssociationSpecificDataSerializerSimple(many=True)
    debt_collection_mandate_id = serializers.ReadOnlyField(source="sepa_mandate.mandate_id")
    debt_collection_mandate_signature_date = serializers.ReadOnlyField(source="sepa_mandate.signature_date")
    debt_collection_mandate_given = serializers.BooleanField(required=False, default=False)

    class Meta:
        model = AssociationMembership
        fields = ('url', 'slug', 'profile', 'association', 'date_joined', 'date_left', 'current', 'new_type', 'pay_by',
                  'debt_collection_mandate_id', 'debt_collection_mandate_given', 'debt_collection_mandate_signature_date',
                  'status', 'type', 'visible_after_date_left', 'specific_data', 'financial_obligations_satisfied',
                  'data_fields_last_modified', 'last_modified')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
            'type': {'lookup_field': 'slug'},
            'new_type': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'status': {'initial': 'Pending'},
        }
        expandable_fields = {
            "association": (AssociationSerializerList, {"read_only": True, "nested": True}),
            'profile': (ProfileSerializer, {"read_only": True}),
            'type': (MemberTypeSerializer, {"read_only": True, "nested": True}),
            'new_type': (MemberTypeSerializer, {"read_only": True, "nested": True}),
        }

    def validate(self, attrs):
        specific_data = attrs.pop('specific_data', None)
        debt_collection_mandate_given = attrs.pop("debt_collection_mandate_given", False)

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Validate membership
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        attrs = super().validate(attrs)

        profile = get_for_patch(self.instance, attrs, 'profile')
        association = get_for_patch(self.instance, attrs, 'association')
        type_ = get_for_patch(self.instance, attrs, 'type')
        old_type = getattr(self.instance, 'type', None)
        new_type = get_for_patch(self.instance, attrs, 'new_type')
        status = get_for_patch(self.instance, attrs, 'status')
        date_left = get_for_patch(self.instance, attrs, 'date_left')
        date_joined = get_for_patch(self.instance, attrs, 'date_joined')
        change_type_on_period_end = get_for_patch(self.instance, attrs, 'change_type_on_period_end')
        pay_by = get_for_patch(self.instance, attrs, 'pay_by')

        user = self.context['request'].user
        board = user.has_perm('board', association)

        errors = {}
        if self.instance is None:
            if status != 'Pending' and status is not None:
                errors["status"] = "Fresh memberships should have status='Pending'"
            if user == profile.user:
                attrs['user'] = profile.user
            else:
                errors["profile"] = "Profile does not belong to user sending the request."

        if date_left and date_left < date_joined:
            errors['date_left'] = "End date must be after start date."

        if pay_by == AssociationMembership.PayBy.DEBT_COLLECTION.value:
            if (self.instance and self.instance.pay_by != AssociationMembership.PayBy.DEBT_COLLECTION.value) or (self.instance is None):
                if not debt_collection_mandate_given:
                    errors["debt_collection_mandate_given"] = "Mandate for debt collection has to be given when paying by debt collection."

        if type_.association != association:
            errors["membertype E_1"] = "Association of membertype does not belong to this association"
        if not board:
            if type_.only_chooseable_by_board:
                errors["membertype E_2"] = "Only the board can assign this membertype"
            if old_type is not None and old_type != type_:
                errors["membertype E_3"] = "Only the board can change current membertype"
        if change_type_on_period_end and new_type is None:
            errors["change_type_on_period_end"] = "You cannot say you change your type, but not have a type to change to"
        if type_.disabled:
            errors["membertype E_4"] = "This is not a valid type, it has been disabled."
        if new_type and new_type.disabled:
            errors["new_membertype"] = "This is not a valid type, it has been disabled."

        if errors:
            raise ValidationError(errors)

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Validate specific data
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\

        if self.instance is None:
            not_supplied_all_mandatory_data_fields = AssociationSpecificDataFields.objects\
                .filter(association=association, mandatory=True)\
                .exclude(slug__in=[data['data_field'].slug for data in specific_data])\
                .exists()
            if not_supplied_all_mandatory_data_fields:
                errors["mandatory"] = "Not all mandatory data fields have been supplied."

        if specific_data:
            for data in specific_data:
                if association != data["data_field"].association:
                    errors[data["data_field"].name] = f"{association.slug}, doesn't match the association, {data['data_field'].association.slug}, of the data_field {data['data_field'].slug}"

                if data["data_field"].type == AssociationSpecificDataFields.Type.BOOLEAN.value:
                    if data["value"] not in ("True", "False"):
                        errors[data["data_field"].name] = "Boolean value has to be either 'True' or 'False'."
                    if data["data_field"].mandatory and not data["value"] == "True" and not board:
                        errors[data["data_field"].name] = "Value has to be 'True', when data field is mandatory."
                elif data["data_field"].type == AssociationSpecificDataFields.Type.CHOICE.value:
                    if data["value"] not in data["data_field"].choices.split(","):
                        errors[data["data_field"].name] = "Boolean value has to be either 'True' or 'False'."

                if data["data_field"].board_only and not board:
                    errors[data["data_field"].name] = "Board only fields can only be set by board members."

        if errors:
            raise ValidationError(errors)

        attrs['specific_data'] = specific_data
        return attrs


class AssociationMembershipSerializerUpdate(AssociationMembershipSerializerBase):
    class Meta(AssociationMembershipSerializerBase.Meta):
        extra_kwargs = {
            **AssociationMembershipSerializerBase.Meta.extra_kwargs,
            'profile': {'lookup_field': 'slug', "read_only": True},
            'association': {'lookup_field': 'slug', "read_only": True},
        }

    def update(self, instance: AssociationMembership, validated_data):
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Update specific data
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        specific_data = instance.specific_data.all()
        supplied_specific_data = validated_data.pop('specific_data')
        if supplied_specific_data and specific_data:
            for supplied_data in supplied_specific_data:
                for data in specific_data:
                    if data.data_field == supplied_data["data_field"]:
                        data.value = supplied_data["value"]
                        break
            AssociationSpecificData.objects.bulk_update(specific_data, ["value"], 50)

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Update sepa mandate
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        if validated_data.get("pay_by", ""):
            if instance.pay_by != AssociationMembership.PayBy.DEBT_COLLECTION and validated_data["pay_by"] == AssociationMembership.PayBy.DEBT_COLLECTION:
                sepa_mandate = SepaMandate.objects.create(
                    mandate_id=uuid.uuid4(),
                    signature_date=datetime.date.today(),
                    association=instance.association,
                    profile=instance.profile,
                )
                instance.sepa_mandate = sepa_mandate
            elif instance.pay_by == AssociationMembership.PayBy.DEBT_COLLECTION and validated_data["pay_by"] != AssociationMembership.PayBy.DEBT_COLLECTION:
                sepa_mandate = instance.sepa_mandate
                sepa_mandate.withdrawal_date = datetime.date.today()
                sepa_mandate.save()
                instance.sepa_mandate = None

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Update membership
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        instance = super().update(instance, validated_data)

        return instance


class AssociationMembershipSerializerCreate(AssociationMembershipSerializerBase):
    def create(self, validated_data):
        specific_data = validated_data.pop('specific_data')
        not_supplied_data_fields = AssociationSpecificDataFields.objects \
            .filter(association=validated_data["association"]) \
            .exclude(slug__in=[specific_datum['data_field'].slug for specific_datum in specific_data]) \
            .all()
        # .filter(association=validated_data["association"], mandatory=True) \

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Create membership and, if needed, sepa mandate
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        if validated_data.get("pay_by", "") == AssociationMembership.PayBy.DEBT_COLLECTION:
            sepa_mandate = SepaMandate.objects.create(
                mandate_id=uuid.uuid4(),
                signature_date=datetime.date.today(),
                association=validated_data["association"],
                profile=validated_data["profile"],
            )
            membership = AssociationMembership.objects.create(sepa_mandate=sepa_mandate, **validated_data)
        else:
            membership = AssociationMembership.objects.create(**validated_data)

        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        #  Create specific data
        #  \\\\\\\\\\\\\\\\\\\\\\\\\\
        if specific_data:
            for data in specific_data:
                data['membership'] = membership
                data['association'] = membership.association
                AssociationSpecificData.objects.create(**data)
        if not_supplied_data_fields:
            for field in not_supplied_data_fields:
                AssociationSpecificData.objects.create(
                    membership=membership,
                    association=membership.association,
                    data_field=field,
                    value=field.default
                )
        return membership
