from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import Group
from apps.MySU.models import User
from tests.tests import fixtures


class GroupTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_grant_board_permissions_helios(self):
        response = self.client.get("/groups/2d9551a0-7e40-461e-aa49-e578eb608b65")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Permissions do not match")
        data = {
            "permissions": ["board"]
        }
        response = self.client.patch("/groups/2d9551a0-7e40-461e-aa49-e578eb608b65", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_grant_board_permissions_student_union(self):
        response = self.client.get("/groups/dd0d1b61-e054-4495-b540-3a6c65a5f610")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Initial permissions do not match")
        data = {
            "permissions": ["board"]
        }
        response = self.client.patch("/groups/dd0d1b61-e054-4495-b540-3a6c65a5f610", data=data)
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Final permissions do not match")

    def test_revoke_board_permissions_helios(self):
        response = self.client.get("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Permissions do not match")
        data = {
            "permissions": []
        }
        response = self.client.patch("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_revoke_board_permissions_student_union(self):
        response = self.client.get("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group["permissions"], "Initial permissions do not match")
        data = {
            "permissions": [],
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data, format="json")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group["permissions"], "Final permissions do not match")

    def test_put_helios(self):
        pass

    def test_put_student_union(self):
        pass

    def test_patch_helios(self):
        response = self.client.get("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("board of Helios", group["full_name"])
        data = {
            "full_name": 'TestCie',
        }
        response = self.client.patch("/groups/c2d3528a-46b3-4830-afa0-dc00831f040a", data=data)
        self.assertEqual(403, response.status_code, response.data)

    def test_patch_student_union(self):
        response = self.client.get("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc")
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("board of Student Union", group["full_name"])
        data = {
            "full_name": 'TestCie',
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data)
        self.assertEqual(200, response.status_code, response.data)
        group = response.data
        self.assertEqual("TestCie", group["full_name"])

    def test_patch_association(self):
        # confirm that we cannot change the association
        data = {
            "association": '/associations/rising-sun-alienship-helios',
        }
        response = self.client.patch("/groups/705f007c-dba5-4da6-9dd7-ba7c22af88bc", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_helios_without_permissions(self):
        data = {
            "association": '/associations/rising-sun-alienship-helios',
            "full_name": "Testcie",
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_helios_with_permissions(self):
        data = {
            "association": '/associations/rising-sun-alienship-helios',
            "full_name": "Testcie",
            "permissions": [
                "board"
            ],
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_student_union_without_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_valid_permissions_2(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [],
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual([], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_valid_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [
                "board"
            ]
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(201, response.status_code, response.data)
        group = response.data
        self.assertEqual(["board"], group['permissions'], "Permissions do not match")

    def test_post_student_union_with_invalid_permissions(self):
        data = {
            "association": "/associations/student-union",
            "full_name": "Testcie",
            "permissions": [
                "change_association"
            ]
        }
        response = self.client.post("/groups", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_get_helios(self):
        response = self.client.get("/groups/218eccab-d65f-4749-8b94-c69872b952ed")
        self.assertEqual(200, response.status_code, response.data)

    def test_get_student_union(self):
        response = self.client.get("/groups/3ce35105-5042-4852-839f-3cc8af77f5ce")
        self.assertEqual(200, response.status_code, response.data)

    def test_list(self):
        response = self.client.get('/groups')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groups".format(response.data))
        self.assertEqual(8, response.data['count'], msg="number of groups doesn't match expected number of groups")

        response = self.client.get('/groups?board_group=True')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all groups".format(response.data))
        self.assertEqual(5, response.data['count'], msg="number of boards doesn't match expected number of boards")

        response = self.client.get('/groups?board_group=False')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all committees".format(response.data))
        self.assertEqual(3, response.data['count'], msg="number of committees doesn't match expected number of committees")

        self.assertEqual(Group.objects.filter(current_board=True).count(), 3, "There are too many current_boards found")

    def test_delete(self):
        response = self.client.delete('/groups/3ce35105-5042-4852-839f-3cc8af77f5ce')
        self.assertEqual(204, response.status_code, response.data)
        response = self.client.delete('/groups/c2d3528a-46b3-4830-afa0-dc00831f040a')
        self.assertEqual(403, response.status_code, response.data)
